@@ -1,5 +1,6 @@
 -- Get data from pg_activity from pg 10 to pg 11
 -- We assume a background_worker with a not null query is a parallel worker.
+-- NEW pg_activity.backend_type
 SELECT
       a.pid AS pid,
       a.application_name AS application_name,
@@ -12,7 +13,8 @@ SELECT
       a.wait_event as wait,
       a.usename AS user,
       a.state AS state,
-      convert_from(a.query::bytea, coalesce(pg_catalog.pg_encoding_to_char(b.encoding), 'UTF8')) AS query,
+      convert_from(replace(a.query, '\', '\\')::bytea, coalesce(pg_catalog.pg_encoding_to_char(b.encoding), 'UTF8')) AS query,
+      NULL AS query_leader_pid,
       (   a.backend_type = 'background worker'
           AND a.query IS NOT NULL
       ) AS is_parallel_worker
