@@ -1,4 +1,5 @@
 -- Get data from pg_activity since pg 11
+-- NEW pg_activity.backend_type value for 'parallel worker'
 SELECT
       a.pid AS pid,
       a.application_name AS application_name,
@@ -11,7 +12,8 @@ SELECT
       a.wait_event AS wait,
       a.usename AS user,
       a.state AS state,
-      convert_from(a.query::bytea, coalesce(pg_catalog.pg_encoding_to_char(b.encoding), 'UTF8')) AS query,
+      convert_from(replace(a.query, '\', '\\')::bytea, coalesce(pg_catalog.pg_encoding_to_char(b.encoding), 'UTF8')) AS query,
+      NULL AS query_leader_pid,
       a.backend_type = 'parallel worker' AS is_parallel_worker
  FROM
       pg_stat_activity a
