@@ -1,5 +1,6 @@
-import optparse
 import time
+from argparse import Namespace
+from functools import partial
 from typing import Dict, List, Optional, cast
 
 import attr
@@ -13,8 +14,7 @@ def main(
     term: Terminal,
     data: Data,
     host: types.Host,
-    options: optparse.Values,
-    dsn: str,
+    options: Namespace,
     *,
     render_header: bool = True,
     render_footer: bool = True,
@@ -26,19 +26,27 @@ def main(
 
     is_local = data.pg_is_local() and data.pg_is_local_access()
 
-    skip_sizes = options.nodbsize
-    pg_db_info = data.pg_get_db_info(
-        None, using_rds=options.rds, skip_sizes=options.nodbsize
+    skip_db_size = options.nodbsize
+    server_information = data.pg_get_server_information(
+        prev_server_info=None,
+        using_rds=options.rds,
+        skip_db_size=options.nodbsize,
+        skip_tempfile=options.notempfiles,
+        skip_walreceiver=options.nowalreceiver,
     )
 
     flag = types.Flag.from_options(is_local=is_local, **vars(options))
     ui = types.UI.make(
         flag=flag,
+        refresh_time=options.refresh,
         min_duration=options.minduration,
         duration_mode=int(options.durationmode),
-        query_display_mode=int(options.querydisplaymode),
-        max_db_length=min(max(int(pg_db_info["max_length"]), 8), 16),
+        wrap_query=options.wrap_query,
+        max_db_length=min(max(server_information.max_dbname_length, 8), 16),
         filters=data.filters,
+        show_instance_info_in_header=options.show_instance_info_in_header,
+        show_worker_info_in_header=options.show_worker_info_in_header,
+        show_system_info_in_header=options.show_system_info_in_header,
     )
 
     key, in_help = None, False
@@ -60,7 +68,7 @@ def main(
             elif not ui.interactive() and key == keys.SPACE:
                 ui.toggle_pause()
             elif options.nodbsize and key == keys.REFRESH_DB_SIZE:
-                skip_sizes = False
+                skip_db_size = False
             elif key is not None:
                 if keys.is_process_next(key):
                     if pg_procs.focus_next():
@@ -85,6 +93,12 @@ def main(
                 elif key.name == keys.CANCEL_SELECTION:
                     pg_procs.reset()
                     ui.end_interactive()
+                elif keys.is_toggle_header_sys_info(key):
+                    ui.toggle_system_info_in_header()
+                elif keys.is_toggle_header_inst_info(key):
+                    ui.toggle_instance_info_in_header()
+                elif keys.is_toggle_header_worker_info(key):
+                    ui.toggle_worker_info_in_header()
                 elif pg_procs.selected and key in (
                     keys.PROCESS_CANCEL,
                     keys.PROCESS_KILL,
@@ -134,9 +148,7 @@ def main(
                     ui.end_interactive()
                     changes = {
                         "duration_mode": handlers.duration_mode(key, ui.duration_mode),
-                        "query_display_mode": handlers.query_display_mode(
-                            key, ui.query_display_mode
-                        ),
+                        "wrap_query": handlers.wrap_query(key, ui.wrap_query),
                     }
                     if key in (keys.REFRESH_TIME_INCREASE, keys.REFRESH_TIME_DECREASE):
                         changes["refresh_time"] = handlers.refresh_time(
@@ -165,19 +177,16 @@ def main(
 
             else:
                 if not ui.in_pause and not ui.interactive():
-                    pg_db_info = data.pg_get_db_info(
-                        pg_db_info, using_rds=options.rds, skip_sizes=skip_sizes
-                    )
-                    if options.nodbsize and not skip_sizes:
-                        skip_sizes = True
+                    if options.nodbsize and not skip_db_size:
+                        skip_db_size = True
 
-                    dbinfo = types.DBInfo(
-                        total_size=int(pg_db_info["total_size"]),
-                        size_ev=int(pg_db_info["size_ev"]),
+                    server_information = data.pg_get_server_information(
+                        prev_server_info=server_information,
+                        using_rds=options.rds,
+                        skip_db_size=skip_db_size,
+                        skip_tempfile=options.notempfiles,
+                        skip_walreceiver=options.nowalreceiver,
                     )
-                    tps = int(pg_db_info["tps"])
-
-                    active_connections = data.pg_get_active_connections()
                     memory, swap, load = activities.mem_swap_load()
                     system_info = types.SystemInfo.default(
                         memory=memory, swap=swap, load=load
@@ -224,17 +233,16 @@ def main(
                     activity_stats = (pg_procs, system_info) if is_local else pg_procs  # type: ignore[assignment]
 
                 if options.output is not None:
+                    custom_asdict = partial(attr.asdict, recurse=True)
                     with open(options.output, "a") as f:
-                        utils.csv_write(f, map(attr.asdict, pg_procs.items))
+                        utils.csv_write(f, map(custom_asdict, pg_procs.items))
 
                 views.screen(
                     term,
                     ui,
                     host=host,
-                    dbinfo=dbinfo,
                     pg_version=data.pg_version,
-                    tps=tps,
-                    active_connections=active_connections,
+                    server_information=server_information,
                     activity_stats=activity_stats,
                     message=msg_pile.get(),
                     render_header=render_header,
